﻿using Connected.Data;

namespace Connected.Net.Endpoints;

/// <summary>
/// Represents a single endpoint in the environment.
/// </summary>
/// <remarks>
/// Each instance should be defined as an endpoint for other instances 
/// to be able to find it on the network and communicate with it.
/// </remarks>
public interface IEndpoint : IPrimaryKey<int>
{
	/// <summary>
	/// The name of the Endpoint. This is descriptive property and has
	/// no special meaning in the environment.
	/// </summary>
	string Name { get; init; }
	/// <summary>
	/// The IP address or Url of the instance where it responds to requests. This value
	/// should be unique across the environment.
	/// </summary>
	string Address { get; init; }
	/// <summary>
	/// The endpoint status. For dynamic scale outs, endpoint could be defined but disabled when
	/// not needed. If scaling is needed this value could automatically go to the <see cref="Status.Enabled"/>.
	/// </summary>
	Status Status { get; init; }
}
